/* ex: set ro ft=c:
 * !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
 *
 * This file is generated automatically from './src/pmc/complex.pmc'
 * by tools/build/pmc2c.pl.
 *
 * Any changes made here will be lost!
 *
 */

/* HEADERIZER HFILE: none */
/* HEADERIZER STOP */

#include "parrot/parrot.h"
#include "parrot/extend.h"
#include "parrot/dynext.h"
#include "pmc_fixedintegerarray.h"
#include "pmc_complex.h"
#include "pmc_default.h"
#include "complex.str"
#include "pmc_continuation.h"
#line 1 "./src/pmc/complex.pmc"
/*
Copyright (C) 2004-2009, Parrot Foundation.
$Id: complex.pmc 41230 2009-09-12 13:37:58Z bacek $

=head1 NAME

src/pmc/complex.pmc - Complex Numbers PMC Class

=head1 DESCRIPTION

C<Complex> provides a representation of complex numbers. It handles
string parsing/generating and basic mathematical operations.

=head2 Functions

=over 4

=cut

Equations used are sometimes listed.  At times, multiple equations are given,
but those starting with => are the ones used

*/

/*

=item C<static void
complex_parse_string(PARROT_INTERP, FLOATVAL *re, FLOATVAL *im, STRING *value)>

Parses the string in C<value> to produce a complex number, represented
by the real (C<*re>) and imaginary (C<*im>) parts. Raises an exception
if it cannot understand the string.  The string should be of the form
C<a+bi> with optional spaces around C<+> and before C<i>. You can also
use C<j> instead of C<i>.

=cut

*/

static void
complex_parse_string(PARROT_INTERP, FLOATVAL *re, FLOATVAL *im, STRING *value) {
    char   * const str        = Parrot_str_to_cstring(interp, value);
    char   *t                 = str;
    char   *first_num_offset  = str;
    char   *second_num_offset = NULL;
    STRING *S;

    INTVAL  i                 = 0;
    INTVAL  first_num_minus   = 0;
    INTVAL  second_num_minus  = 0;
    UINTVAL first_num_length, second_num_length;

    /* walk the string and identify the real and imaginary parts */

    if (*t == '-') {
        /* first number is negative */
        t++;
        first_num_minus = 1;

        /* allow for an optional space */
        if (*t == ' ')
            t++;
        first_num_offset = t;
    }

    /* skip digits */
    while (*t >= '0' && *t <= '9')
        t++;

    if (*t == '.') {
        /* this number has a decimal point */
        t++;

        /* skip digits */
        while (*t >= '0' && *t <= '9')
            t++;
    }

    /* save the length of the real part */
    first_num_length = t - first_num_offset;

    /* end of string; we only have a real part */
    if (*t == 0) {
        second_num_length = 0;
    }
    else if ((*t == 'i' || *t == 'j') && *(t+1) == 0) {
        /* there is only an imaginary part, so the first number was
            actually the imaginary part */
        second_num_length = first_num_length;
        first_num_length  = 0;
        second_num_offset = first_num_offset;
        second_num_minus  = first_num_minus;
        first_num_minus   = 0;

        /* this is useful if there is no number for
            the imaginary part, like in "-i" */
        i = 1;
    }
    else {
        /* skip an optional space */
        if (*t == ' ')
            t++;

        /* expect "+" or "-" and the imaginary part */
        if (*t == '+' || *t == '-') {
            /* save the sign */
            second_num_minus = (*t == '-');
            t++;

            /* skip another optional space */
            if (*t == ' ')
                t++;

            /* save the beginning of the imaginary part */
            second_num_offset = t;

            /* skip digits */
            while (*t >= '0' && *t <= '9')
                t++;

            if (*t == '.') {
                /* this number has a decimal point */
                t++;

                /* skip digits */
                while (*t >= '0' && *t <= '9')
                    t++;
            }

            /* save the length of the imaginary part */
            second_num_length = t - second_num_offset;

            /* allow for one more optional space */
            if (*t == ' ')
                t++;

            /* verify that the string ends properly */
            if ((*t != 'i' && *t != 'j') || (*(t+1) != 0)) {
                /* imaginary part does not end in 'i' or 'j' */
                Parrot_ex_throw_from_c_args(interp, NULL,
                    EXCEPTION_INVALID_STRING_REPRESENTATION,
                    "Complex: malformed string");
            }

            /* this is useful if there is no number for the
                imaginary part, like in "2+i" */
            i = 1;

            /* all is OK, save the number */
        }
        else {
            /* "+" or "-" not found: error */
            Parrot_str_free_cstring(str);

            Parrot_ex_throw_from_c_args(interp, NULL,
                EXCEPTION_INVALID_STRING_REPRESENTATION,
                "Complex: malformed string");
        }
    }

    /* now we have the offsets and the lengths we turn them into float values */

    if (first_num_length) {
        /* there is a real part, interpret it */
        S   = Parrot_str_new(interp, first_num_offset, first_num_length);
        *re = Parrot_str_to_num(interp, S);
    }
    else {
        /* consider the real part 0.0 */
        *re = 0.0;
    }

    if (second_num_length) {
        /* there is an imaginary part, interpret it */
        S   = Parrot_str_new(interp, second_num_offset, second_num_length);
        *im = Parrot_str_to_num(interp, S);
    }
    else {
        /* consider the imaginary part 0.0 */
        if (i) /* the string was something like "1+i" */
            *im = 1.0;
        else
            *im = 0.0;
    }

    if (first_num_minus)
        *re = -*re;

    if (second_num_minus)
        *im = -*im;

    Parrot_str_free_cstring(str);
}

static void
int_check_divide_zero(PARROT_INTERP, INTVAL value) {
    if (value == 0)
        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_DIV_BY_ZERO,
            "Divide by zero");
}

static void
float_check_divide_zero(PARROT_INTERP, FLOATVAL value) {
    if (FLOAT_IS_ZERO(value))
        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_DIV_BY_ZERO,
            "Divide by zero");
}

static void
complex_check_divide_zero(PARROT_INTERP, PMC *value) {
    /* Throw an exception if we are dividing by zero. Check both the real part
     * and the imaginary part.*/
    if (FLOAT_IS_ZERO(VTABLE_get_number_keyed_int(interp, value, 0))
            && FLOAT_IS_ZERO(VTABLE_get_number_keyed_int(interp, value, 1)))
        Parrot_ex_throw_from_c_args(interp, NULL, EXCEPTION_DIV_BY_ZERO,
            "Divide by zero");
}
#line 240 "./src/pmc/complex.c"
static  PMC  * Parrot_Complex_absolute(PARROT_INTERP, PMC *pmc, PMC *dest);
static  PMC * Parrot_Complex_add(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_add_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  PMC  * Parrot_Complex_clone(PARROT_INTERP, PMC *pmc);
static  PMC * Parrot_Complex_divide(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  INTVAL  Parrot_Complex_get_bool(PARROT_INTERP, PMC *pmc);
static  INTVAL  Parrot_Complex_get_integer(PARROT_INTERP, PMC *pmc);
static  INTVAL  Parrot_Complex_get_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key);
static  INTVAL  Parrot_Complex_get_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key);
static  FLOATVAL  Parrot_Complex_get_number(PARROT_INTERP, PMC *pmc);
static  FLOATVAL  Parrot_Complex_get_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key);
static  FLOATVAL  Parrot_Complex_get_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key);
static  FLOATVAL  Parrot_Complex_get_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key);
static  PMC  * Parrot_Complex_get_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key);
static  PMC  * Parrot_Complex_get_pmc_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key);
static  PMC  * Parrot_Complex_get_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key);
static  STRING  * Parrot_Complex_get_string(PARROT_INTERP, PMC *pmc);
static  void  Parrot_Complex_i_absolute(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_i_add(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_add_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void Parrot_Complex_i_divide(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void Parrot_Complex_i_multiply(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void  Parrot_Complex_i_neg(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_i_subtract(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_i_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_i_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void  Parrot_Complex_init(PARROT_INTERP, PMC *pmc);
static  void  Parrot_Complex_init_pmc(PARROT_INTERP, PMC *pmc, PMC *initializer);
static  opcode_t  * Parrot_Complex_invoke(PARROT_INTERP, PMC *pmc, void *next);
static  INTVAL Parrot_Complex_is_equal(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC * Parrot_Complex_multiply(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  PMC  * Parrot_Complex_neg(PARROT_INTERP, PMC *pmc, PMC *dest);
static  PMC * Parrot_Complex_pow(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_set_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key, INTVAL value);
static  void  Parrot_Complex_set_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, INTVAL value);
static  void  Parrot_Complex_set_integer_native(PARROT_INTERP, PMC *pmc, INTVAL value);
static  void  Parrot_Complex_set_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key, FLOATVAL value);
static  void  Parrot_Complex_set_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key, FLOATVAL v);
static  void  Parrot_Complex_set_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, FLOATVAL value);
static  void  Parrot_Complex_set_number_native(PARROT_INTERP, PMC *pmc, FLOATVAL value);
static  void  Parrot_Complex_set_pmc(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_set_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key, PMC *value);
static  void  Parrot_Complex_set_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, PMC *value);
static  void  Parrot_Complex_set_string_native(PARROT_INTERP, PMC *pmc, STRING *value);
static  PMC * Parrot_Complex_subtract(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest);
static  PMC  * Parrot_Complex_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest);
static  PMC  * Parrot_Complex_multi_add_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_add_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_add_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_add_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC  * Parrot_Complex_multi_subtract_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_subtract_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_subtract_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_subtract_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC  * Parrot_Complex_multi_multiply_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_multiply_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_multiply_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_multiply_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  PMC  * Parrot_Complex_multi_divide_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_divide_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void  Parrot_Complex_multi_i_divide_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  void  Parrot_Complex_multi_i_divide_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  INTVAL  Parrot_Complex_multi_is_equal_Complex(PARROT_INTERP, PMC *pmc, PMC *value);
static  INTVAL  Parrot_Complex_multi_is_equal_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value);
static  void Parrot_Complex_nci_ln(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_exp(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sin(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_cos(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_tan(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_cot(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sec(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_csc(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asin(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acos(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_atan(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acot(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acsc(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asec(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sinh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_cosh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_tanh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_coth(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_csch(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_sech(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asinh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acosh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_atanh(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acoth(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_acsch(PARROT_INTERP, PMC *pmc);
static  void Parrot_Complex_nci_asech(PARROT_INTERP, PMC *pmc);
static  PMC  * Parrot_Complex_multi_pow_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  PMC  * Parrot_Complex_multi_pow_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest);
static  void Parrot_Complex_nci_sqrt(PARROT_INTERP, PMC *pmc);
void    Parrot_Complex_class_init(PARROT_INTERP, int, int);
PARROT_EXPORT VTABLE* Parrot_Complex_update_vtable(ARGMOD(VTABLE*));
PARROT_EXPORT VTABLE* Parrot_Complex_get_vtable(PARROT_INTERP);
void    Parrot_Complex_ro_class_init(PARROT_INTERP, int, int);
PARROT_EXPORT VTABLE* Parrot_Complex_ro_update_vtable(ARGMOD(VTABLE*));
PARROT_EXPORT VTABLE* Parrot_Complex_ro_get_vtable(PARROT_INTERP);
static  PMC  *
Parrot_Complex_absolute(PARROT_INTERP, PMC *pmc, PMC *dest)
{
#line 1161 "./src/pmc/complex.pmc"

    FLOATVAL re, im, d;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    d = sqrt(re*re + im*im);

    dest = pmc_new(interp,
        Parrot_get_ctx_HLL_type(interp, enum_class_Float));

    VTABLE_set_number_native(interp, dest, d);
    return dest;
#line 366 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_add(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_add_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_add_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval = PMCNULL;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "add", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 668 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re + value);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 406 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_add_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 664 "./src/pmc/complex.pmc"

    return VTABLE_add_float(interp, pmc, (FLOATVAL)value, dest);
#line 414 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_clone(PARROT_INTERP, PMC *pmc)
{
#line 318 "./src/pmc/complex.pmc"

    PMC * const dest = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 430 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_divide(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_divide_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_divide_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval = PMCNULL;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "divide", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 998 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    float_check_divide_zero(interp, value);
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, re / value);
    SETATTR_Complex_im(interp, dest, im / value);

    return dest;
#line 472 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 984 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    int_check_divide_zero(interp, value);
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, re / value);
    SETATTR_Complex_im(interp, dest, im / value);

    return dest;
#line 490 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_bool(PARROT_INTERP, PMC *pmc)
{
#line 371 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    return !(FLOAT_IS_ZERO(re) && FLOAT_IS_ZERO(im));
#line 501 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_integer(PARROT_INTERP, PMC *pmc)
{
#line 352 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number(interp, pmc);
    return (INTVAL)f;
#line 510 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key)
{
#line 398 "./src/pmc/complex.pmc"

    STRING * const s = VTABLE_get_string(interp, key);
    return VTABLE_get_integer_keyed_str(interp, pmc, s);
#line 519 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_get_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key)
{
#line 403 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number_keyed_str(interp, pmc, key);
    return (INTVAL)f;
#line 528 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number(PARROT_INTERP, PMC *pmc)
{
#line 357 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    return sqrt(re * re + im * im);
#line 539 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key)
{
#line 408 "./src/pmc/complex.pmc"

    STRING * const s = VTABLE_get_string(interp, key);
    return VTABLE_get_number_keyed_str(interp, pmc, s);
#line 548 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key)
{
#line 479 "./src/pmc/complex.pmc"

    FLOATVAL f;
    switch (key) {
        case 0:
            GETATTR_Complex_re(interp, pmc, f);
            break;
        case 1:
            GETATTR_Complex_im(interp, pmc, f);
            break;
        default:
            Parrot_ex_throw_from_c_args(interp, NULL,
                EXCEPTION_INVALID_OPERATION, "Complex: key must be 0 or 1");
    }
    return f;
#line 568 "./src/pmc/complex.c"
}
static  FLOATVAL 
Parrot_Complex_get_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key)
{
#line 413 "./src/pmc/complex.pmc"

    FLOATVAL value;
    if (Parrot_str_equal(interp, key, CONST_STRING(interp, "real"))) {
        GETATTR_Complex_re(interp, pmc, value);
    }
    else if (Parrot_str_equal(interp, key, CONST_STRING(interp, "imag"))) {
        GETATTR_Complex_im(interp, pmc, value);
    }
    else
        Parrot_ex_throw_from_c_args(interp, NULL,
            EXCEPTION_INVALID_OPERATION, "Complex: key is neither 'real' or 'imag'");
    return value;
#line 586 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_get_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key)
{
#line 427 "./src/pmc/complex.pmc"

    if (VTABLE_isa(interp, key, CONST_STRING(interp, "Integer"))) {
        const INTVAL i = VTABLE_get_integer(interp, key);
        return VTABLE_get_pmc_keyed_int(interp, pmc, i);
    }
    else {
        STRING * const s = VTABLE_get_string(interp, key);
        return VTABLE_get_pmc_keyed_str(interp, pmc, s);
    }
#line 601 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_get_pmc_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key)
{
#line 455 "./src/pmc/complex.pmc"

    PMC * const    ret = pmc_new(interp, enum_class_Float);
    const FLOATVAL val = VTABLE_get_number_keyed_int(interp, pmc, key);
    VTABLE_set_number_native(interp, ret, val);
    return ret;
#line 612 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_get_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key)
{
#line 438 "./src/pmc/complex.pmc"

    PMC * const    ret = pmc_new(interp, enum_class_Float);
    const FLOATVAL val = VTABLE_get_number_keyed_str(interp, pmc, key);
    VTABLE_set_number_native(interp, ret, val);
    return ret;
#line 623 "./src/pmc/complex.c"
}
static  STRING  *
Parrot_Complex_get_string(PARROT_INTERP, PMC *pmc)
{
#line 364 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    return Parrot_sprintf_c(interp, "%vg%+vgi", re, im);
#line 634 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_absolute(PARROT_INTERP, PMC *pmc)
{
#line 1174 "./src/pmc/complex.pmc"

    FLOATVAL re, im, d;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    d = sqrt(re*re + im*im);
    pmc_reuse(interp, pmc, enum_class_Float, 0);
    VTABLE_set_number_native(interp, pmc, d);
#line 647 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_add(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_add_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_add_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_add", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_add_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 704 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re + value);
#line 682 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_add_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 700 "./src/pmc/complex.pmc"

    VTABLE_i_add_float(interp, pmc, (FLOATVAL)value);
#line 690 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_divide(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_divide_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_divide_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_divide", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_divide_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 1063 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    float_check_divide_zero(interp, value);

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, pmc, re / value);
    SETATTR_Complex_im(interp, pmc, im / value);
#line 729 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_divide_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 1052 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    int_check_divide_zero(interp, value);

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, pmc, re / value);
    SETATTR_Complex_im(interp, pmc, im / value);
#line 744 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_multiply(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_multiply_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_multiply_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_multiply", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 899 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, re * value);
    SETATTR_Complex_im(interp, pmc, im * value);
#line 781 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 890 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, re * value);
    SETATTR_Complex_im(interp, pmc, im * value);
#line 794 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_neg(PARROT_INTERP, PMC *pmc)
{
#line 1099 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, -re);
    SETATTR_Complex_im(interp, pmc, -im);
#line 806 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_i_subtract(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            Parrot_Complex_multi_i_subtract_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                Parrot_Complex_multi_i_subtract_DEFAULT(interp, pmc, value);
            else {
                
                Parrot_mmd_multi_dispatch_from_c_args(interp, "i_subtract", "PP->", pmc, value);
                return;
            }
            break;

    }
}
static  void 
Parrot_Complex_i_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 793 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re - value);
#line 841 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_i_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 789 "./src/pmc/complex.pmc"

    VTABLE_i_subtract_float(interp, pmc, (FLOATVAL) value);
#line 849 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_init(PARROT_INTERP, PMC *pmc)
{
#line 281 "./src/pmc/complex.pmc"

    SETATTR_Complex_re(interp, pmc, 0.0);
    SETATTR_Complex_im(interp, pmc, 0.0);
#line 858 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_init_pmc(PARROT_INTERP, PMC *pmc, PMC *initializer)
{
#line 286 "./src/pmc/complex.pmc"

    const INTVAL arg_type = VTABLE_type(interp, initializer);
    VTABLE_init(interp, pmc);
    switch (arg_type) {
        case enum_class_String:
            VTABLE_set_string_native(interp, pmc, VTABLE_get_string(interp, initializer));
            break;
        case enum_class_FixedFloatArray:
        case enum_class_ResizableFloatArray:
        case enum_class_FixedIntegerArray:
        case enum_class_ResizableIntegerArray:
            if (VTABLE_get_integer(interp, initializer) == 2) {
                FLOATVAL re = VTABLE_get_number_keyed_int(interp, initializer, 0);
                FLOATVAL im = VTABLE_get_number_keyed_int(interp, initializer, 1);
                SETATTR_Complex_re(interp, pmc, re);
                SETATTR_Complex_im(interp, pmc, im);
                break;
            }
            /* else let it fall to default */
        default:
            if (VTABLE_isa(interp, initializer, CONST_STRING(interp, "String"))) {
                STRING * s = VTABLE_get_string(interp, initializer);
                VTABLE_set_string_native(interp, pmc, s);
            }
            else {
                Parrot_ex_throw_from_c_args(interp, NULL,
                    EXCEPTION_INVALID_OPERATION,
                    "Invalid Complex initializer");
            }
    }
#line 894 "./src/pmc/complex.c"
}
static  opcode_t  *
Parrot_Complex_invoke(PARROT_INTERP, PMC *pmc, void *next)
{
#line 235 "./src/pmc/complex.pmc"

    const int   argcP = REG_INT(interp, 3);
    PMC * const res   = pmc_new(interp, enum_class_Complex);

    if (argcP == 1) {
        PMC * const arg = REG_PMC(interp, 5);

        if (arg->vtable->base_type == enum_class_String)
            VTABLE_set_string_native(interp, res, VTABLE_get_string(interp, arg));
        else
            SETATTR_Complex_re(interp, res, VTABLE_get_number(interp, arg));
    }
    else if (argcP == 2) {
        SETATTR_Complex_re(interp, res, VTABLE_get_number(interp, REG_PMC(interp, 5)));
        SETATTR_Complex_im(interp, res, VTABLE_get_number(interp, REG_PMC(interp, 6)));
    }

    REG_PMC(interp, 5) = res;
    return (opcode_t *)next;
#line 919 "./src/pmc/complex.c"
}
static  INTVAL
Parrot_Complex_is_equal(PARROT_INTERP, PMC *pmc, PMC *value)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_is_equal_Complex(interp, pmc, value);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_is_equal_DEFAULT(interp, pmc, value);
            else {
                INTVAL retval;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "is_equal", "PP->I", pmc, value, &retval);
                return retval;
            }
            break;

    }
}
static  PMC *
Parrot_Complex_multiply(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_multiply_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_multiply_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval = PMCNULL;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "multiply", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_multiply_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 858 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re * value);
    SETATTR_Complex_im(interp, dest, im * value);

    return dest;
#line 983 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multiply_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 854 "./src/pmc/complex.pmc"

    return VTABLE_multiply_float(interp, pmc, (FLOATVAL) value, dest);
#line 991 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_neg(PARROT_INTERP, PMC *pmc, PMC *dest)
{
#line 1086 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, -re);
    SETATTR_Complex_im(interp, dest, -im);

    return dest;
#line 1008 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_pow(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_pow_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_pow_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval = PMCNULL;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "pow", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  void 
Parrot_Complex_set_integer_keyed(PARROT_INTERP, PMC *pmc, PMC *key, INTVAL value)
{
#line 581 "./src/pmc/complex.pmc"

    VTABLE_set_number_keyed(interp, pmc, key, (FLOATVAL)value);
#line 1040 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_integer_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, INTVAL value)
{
#line 585 "./src/pmc/complex.pmc"

    VTABLE_set_number_keyed_str(interp, pmc, key, (FLOATVAL)value);
#line 1048 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_integer_native(PARROT_INTERP, PMC *pmc, INTVAL value)
{
#line 551 "./src/pmc/complex.pmc"

    VTABLE_set_number_native(interp, pmc, (FLOATVAL)value);
#line 1056 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_keyed(PARROT_INTERP, PMC *pmc, PMC *key, FLOATVAL value)
{
#line 589 "./src/pmc/complex.pmc"

    if (VTABLE_isa(interp, key, CONST_STRING(interp, "Integer"))) {
        const INTVAL i = VTABLE_get_integer(interp, key);
        VTABLE_set_number_keyed_int(interp, pmc, i, value);
    }
    else {
        STRING *s = VTABLE_get_string(interp, key);
        VTABLE_set_number_keyed_str(interp, pmc, s, value);
    }
#line 1071 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_keyed_int(PARROT_INTERP, PMC *pmc, INTVAL key, FLOATVAL v)
{
#line 495 "./src/pmc/complex.pmc"

    switch (key) {
        case 0:
            SETATTR_Complex_re(interp, pmc, v);
            break;
        case 1:
            SETATTR_Complex_im(interp, pmc, v);
            break;
        default:
            Parrot_ex_throw_from_c_args(interp, NULL,
                EXCEPTION_INVALID_OPERATION, "Complex: key must be 0 or 1");
    }
#line 1089 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, FLOATVAL value)
{
#line 600 "./src/pmc/complex.pmc"

    if (Parrot_str_equal(interp, key, CONST_STRING(interp, "real"))) {
        SETATTR_Complex_re(interp, pmc, value);
    }
    else if (Parrot_str_equal(interp, key, CONST_STRING(interp, "imag"))) {
        SETATTR_Complex_im(interp, pmc, value);
    }
    else
        Parrot_ex_throw_from_c_args(interp, NULL,
            EXCEPTION_INVALID_OPERATION, "Complex: key is neither 'real' or 'imag'");
#line 1105 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_number_native(PARROT_INTERP, PMC *pmc, FLOATVAL value)
{
#line 555 "./src/pmc/complex.pmc"

    SETATTR_Complex_re(interp, pmc, value);
    SETATTR_Complex_im(interp, pmc, 0.0);
#line 1114 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_pmc(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 539 "./src/pmc/complex.pmc"

    if (VTABLE_isa(interp, value, CONST_STRING(interp, "Complex"))) {
        FLOATVAL re, im;
        GETATTR_Complex_re(interp, value, re);
        GETATTR_Complex_im(interp, value, im);
        SETATTR_Complex_re(interp, pmc, re);
        SETATTR_Complex_im(interp, pmc, im);
    }
    else
        VTABLE_set_string_native(interp, pmc, VTABLE_get_string(interp, value));
#line 1130 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_pmc_keyed(PARROT_INTERP, PMC *pmc, PMC *key, PMC *value)
{
#line 612 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number(interp, value);
    VTABLE_set_number_keyed(interp, pmc, key, f);
#line 1139 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_pmc_keyed_str(PARROT_INTERP, PMC *pmc, STRING *key, PMC *value)
{
#line 617 "./src/pmc/complex.pmc"

    const FLOATVAL f = VTABLE_get_number(interp, value);
    VTABLE_set_number_keyed_str(interp, pmc, key, f);
#line 1148 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_set_string_native(PARROT_INTERP, PMC *pmc, STRING *value)
{
#line 532 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    complex_parse_string(interp, &re, &im, value);
    SETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_im(interp, pmc, im);
#line 1159 "./src/pmc/complex.c"
}
static  PMC *
Parrot_Complex_subtract(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
    INTVAL type = VTABLE_type(interp, value);
    /* For dynpmc fallback to MMD */
    if ((type >= enum_class_core_max) || (VTABLE_type(interp, pmc) >= enum_class_core_max))
        type = enum_class_core_max;
    switch(type) {
        case enum_class_Complex:
            return Parrot_Complex_multi_subtract_Complex_PMC(interp, pmc, value, dest);
            break;

        default:
            if (type < enum_class_core_max)
                return Parrot_Complex_multi_subtract_DEFAULT_PMC(interp, pmc, value, dest);
            else {
                PMC *retval = PMCNULL;
                Parrot_mmd_multi_dispatch_from_c_args(interp, "subtract", "PPP->P", pmc, value, dest, &retval);
                return retval;
            }
            break;

    }
}
static  PMC  *
Parrot_Complex_subtract_float(PARROT_INTERP, PMC *pmc, FLOATVAL value, PMC *dest)
{
#line 757 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re - value);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1199 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_subtract_int(PARROT_INTERP, PMC *pmc, INTVAL value, PMC *dest)
{
#line 753 "./src/pmc/complex.pmc"

    return VTABLE_subtract_float(interp, pmc, (FLOATVAL)value, dest);
#line 1207 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_add_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 636 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, dest, self_re + val_re);
    SETATTR_Complex_im(interp, dest, self_im + val_im);

    return dest;
#line 1227 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_add_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 652 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re + VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1243 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_add_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 680 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, pmc, self_re + val_re);
    SETATTR_Complex_im(interp, pmc, self_im + val_im);
#line 1260 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_add_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 693 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re + VTABLE_get_number(interp, value));
#line 1271 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_subtract_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 725 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, dest, self_re - val_re);
    SETATTR_Complex_im(interp, dest, self_im - val_im);

    return dest;
#line 1291 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_subtract_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 741 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re - VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1307 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_subtract_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 769 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    SETATTR_Complex_re(interp, pmc, self_re - val_re);
    SETATTR_Complex_im(interp, pmc, self_im - val_im);
#line 1324 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_subtract_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 782 "./src/pmc/complex.pmc"

    FLOATVAL re;

    GETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_re(interp, pmc, re - VTABLE_get_number(interp, value));
#line 1335 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_multiply_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 827 "./src/pmc/complex.pmc"

    FLOATVAL a, b, c, d;

    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, a);
    GETATTR_Complex_im(interp, pmc, b);
    GETATTR_Complex_re(interp, value, c);
    GETATTR_Complex_im(interp, value, d);
    SETATTR_Complex_re(interp, dest, a * c - b * d);
    SETATTR_Complex_im(interp, dest, a * d + b * c);

    return dest;
#line 1354 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_multiply_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 842 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, dest, re * VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, dest, im * VTABLE_get_number(interp, value));

    return dest;
#line 1370 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_multiply_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 870 "./src/pmc/complex.pmc"

    FLOATVAL a, b, c, d;

    GETATTR_Complex_re(interp, pmc, a);
    GETATTR_Complex_im(interp, pmc, b);
    GETATTR_Complex_re(interp, value, c);
    GETATTR_Complex_im(interp, value, d);
    SETATTR_Complex_re(interp, pmc, a * c - b * d);
    SETATTR_Complex_im(interp, pmc, a * d + b * c);
#line 1385 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_multiply_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 881 "./src/pmc/complex.pmc"

    FLOATVAL re, im;

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, pmc, re * VTABLE_get_number(interp, value));
    SETATTR_Complex_im(interp, pmc, im * VTABLE_get_number(interp, value));
#line 1398 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_divide_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 938 "./src/pmc/complex.pmc"

    FLOATVAL mod, re, im;
    FLOATVAL self_re, self_im, val_re, val_im;

    complex_check_divide_zero(interp, value);
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);
    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    /* a little speed optimisation: cache an intermediate number;
        I'm not sure the compiler does this */

    if (self_im == 0.0 && val_im == 0.0) {
        re = self_re / val_re;
        im = 0.0;
    }
    else {
        mod = (val_re * val_re + val_im * val_im);
        re  = (self_re * val_re + self_im * val_im) / mod;
        im  = (self_im * val_re - self_re * val_im) / mod;
    }

    SETATTR_Complex_re(interp, dest, re);
    SETATTR_Complex_im(interp, dest, im);

    return dest;
#line 1433 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_divide_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 969 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    const FLOATVAL d = VTABLE_get_number(interp, value);
    float_check_divide_zero(interp, d);
    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, dest, re / d);
    SETATTR_Complex_im(interp, dest, im / d);

    return dest;
#line 1452 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_divide_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1012 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    FLOATVAL self_re, self_im, val_re, val_im;

    complex_check_divide_zero(interp, value);

    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);
    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);

    if (self_im == 0.0 && val_im == 0.0) {
        re = self_re / val_re;
        im = 0.0;
    }
    else {
        /* a little speed optimisation: cache an intermediate number;
           I'm not sure the compiler does this */
        const FLOATVAL mod = (val_re * val_re + val_im * val_im);
        re  = (self_re * val_re + self_im * val_im) / mod;
        im  = (self_im * val_re - self_re * val_im) / mod;
    }

    SETATTR_Complex_re(interp, pmc, re);
    SETATTR_Complex_im(interp, pmc, im);

#line 1484 "./src/pmc/complex.c"
}
static  void 
Parrot_Complex_multi_i_divide_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1040 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    const FLOATVAL d = VTABLE_get_number(interp, value);
    float_check_divide_zero(interp, d);

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, pmc, re / d);
    SETATTR_Complex_im(interp, pmc, im / d);
#line 1500 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_multi_is_equal_Complex(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1117 "./src/pmc/complex.pmc"

    FLOATVAL self_re, self_im, val_re, val_im;
    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);
    GETATTR_Complex_re(interp, value, val_re);
    GETATTR_Complex_im(interp, value, val_im);
    return (INTVAL)(self_re == val_re && self_im == val_im);
#line 1513 "./src/pmc/complex.c"
}
static  INTVAL 
Parrot_Complex_multi_is_equal_DEFAULT(PARROT_INTERP, PMC *pmc, PMC *value)
{
#line 1126 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    if (im != 0.0)
        return (INTVAL)0;

    return (re == VTABLE_get_number(interp, value));
#line 1528 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_ln(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 1566 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 1582 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1204 "./src/pmc/complex.pmc"

    PMC * const d  = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, result_re, result_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    /* This is necessary for atan2 to behave */
    if (im == -0.0)
        im = 0.0;

    result_re = log(sqrt(re*re + im*im));
    if (re == 0.0 && im == 0.0) /* atan2(0, 0) not portable */
        result_im = 0.0;
    else
        result_im = atan2(im, re);

    SETATTR_Complex_re(interp, d, result_re);
    SETATTR_Complex_im(interp, d, result_im);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 1613 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto ln_returns;
    /*END RETURN PMC *d */
    }
#line 1223 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
ln_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 1658 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_exp(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 1696 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 1712 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1238 "./src/pmc/complex.pmc"

    PMC * const d  = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, f;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    f  = exp(re);

    SETATTR_Complex_re(interp, d, f * cos(im));

    /* If only sin(pi) worked. */
    if (im == 4.0 * atan(1.0)) {
        SETATTR_Complex_im(interp, d, 0.0);
    }
    else {
        SETATTR_Complex_im(interp, d, f * sin(im));
    }

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 1742 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto exp_returns;
    /*END RETURN PMC *d */
    }
#line 1256 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
exp_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 1787 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sin(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 1825 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 1841 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1292 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, result_re, result_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    if (FLOAT_IS_ZERO(im)) {
        result_re = sin(re);
        result_im = 0.0;
    }
    else if (FLOAT_IS_ZERO(re)) {
        result_re = 0.0;
        result_im = sinh(im);
    }
    else {
        result_re = sin(re) * cosh(im);

        if (im == -0.0)
            result_im = 0.0;
        else
            result_im = cos(re) * sinh(im);

    }

    SETATTR_Complex_re(interp, d, result_re);
    SETATTR_Complex_im(interp, d, result_im);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 1880 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto sin_returns;
    /*END RETURN PMC *d */
    }
#line 1319 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
sin_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 1925 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_cos(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 1963 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 1979 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1322 "./src/pmc/complex.pmc"

    PMC * const d  = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, result_re, result_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    if (FLOAT_IS_ZERO(re)) {
        result_re = cosh(im);
        result_im = 0.0;
    }
    else if (FLOAT_IS_ZERO(im)) {
        result_re = cos(re);
        result_im = 0.0;
    }
    else {
        result_re = cos(re) * cosh(im);
        result_im = -1.0 * sin(re) * sinh(im);
    }

    SETATTR_Complex_re(interp, d, result_re);
    SETATTR_Complex_im(interp, d, result_im);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2013 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto cos_returns;
    /*END RETURN PMC *d */
    }
#line 1344 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
cos_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2058 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_tan(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 2096 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2112 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1347 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 2124 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "sin"), "->P", &d);
#line 1351 "./src/pmc/complex.pmc"

    #line 2128 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "cos"), "->P", &e);
#line 1352 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2139 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto tan_returns;
    /*END RETURN PMC *d */
    }
#line 1356 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
tan_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2184 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_cot(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 2222 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2238 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1359 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));
    #line 2249 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "cos"), "->P", &d);
#line 1362 "./src/pmc/complex.pmc"

    #line 2253 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "sin"), "->P", &e);
#line 1363 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2264 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto cot_returns;
    /*END RETURN PMC *d */
    }
#line 1367 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
cot_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2309 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sec(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 2347 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2363 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1370 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));
    SETATTR_Complex_re(interp, d, 1.0);
    SETATTR_Complex_im(interp, d, 0.0);
    #line 2376 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "cos"), "->P", &e);
#line 1375 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2387 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto sec_returns;
    /*END RETURN PMC *d */
    }
#line 1379 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
sec_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2432 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_csc(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 2470 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2486 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1382 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    SETATTR_Complex_re(interp, d, 1.0);
    SETATTR_Complex_im(interp, d, 0.0);

    #line 2501 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "sin"), "->P", &e);
#line 1389 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2512 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto csc_returns;
    /*END RETURN PMC *d */
    }
#line 1393 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
csc_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2557 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asin(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 2595 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2611 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1429 "./src/pmc/complex.pmc"

    FLOATVAL d_re, d_im, e_re, e_im, self_re, self_im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *       e = pmc_new(interp, VTABLE_type(interp, pmc));
    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    e      = Parrot_Complex_multi_multiply_Complex_PMC(interp, pmc, pmc, e);
    GETATTR_Complex_re(interp, e, e_re);
    GETATTR_Complex_im(interp, e, e_im);
    SETATTR_Complex_re(interp, e, 1.0 - e_re);
    SETATTR_Complex_im(interp, e, -e_im);

    #line 2632 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, e, CONST_STRING_GEN(interp, "sqrt"), "->P", &d);
#line 1442 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);
    SETATTR_Complex_re(interp, d, d_re - self_im);
    SETATTR_Complex_im(interp, d, d_im + self_re);

    #line 2641 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "ln"), "->P", &d);
#line 1448 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);
    SETATTR_Complex_re(interp, e, d_im);
    SETATTR_Complex_im(interp, e, d_re ? -d_re : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 2654 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto asin_returns;
    /*END RETURN PMC *e */
    }
#line 1454 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
asin_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2699 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acos(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 2737 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2753 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1457 "./src/pmc/complex.pmc"

    FLOATVAL d_re, d_im, e_re, e_im, self_re, self_im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *       e = pmc_new(interp, VTABLE_type(interp, pmc));
    GETATTR_Complex_re(interp, pmc, self_re);
    GETATTR_Complex_im(interp, pmc, self_im);

    e = Parrot_Complex_multi_multiply_Complex_PMC(interp, pmc, pmc, e);
    GETATTR_Complex_re(interp, e, e_re);
    GETATTR_Complex_im(interp, e, e_im);
    SETATTR_Complex_re(interp, e, 1.0 - e_re);
    SETATTR_Complex_im(interp, e, -e_im);

    #line 2774 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, e, CONST_STRING_GEN(interp, "sqrt"), "->P", &d);
#line 1470 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);
    SETATTR_Complex_re(interp, d, d_re + self_im);
    SETATTR_Complex_im(interp, d, d_im - self_re);

    #line 2783 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "ln"), "->P", &e);
#line 1476 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, e, e_re);
    GETATTR_Complex_im(interp, e, e_im);
    SETATTR_Complex_re(interp, d, e_im + 2.0 * atan(1.0));
    SETATTR_Complex_im(interp, d, e_re ? -e_re : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 2796 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acos_returns;
    /*END RETURN PMC *d */
    }
#line 1482 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acos_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2841 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_atan(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 2879 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 2895 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1485 "./src/pmc/complex.pmc"

    PMC * const d  = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e  = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im, d_re, d_im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d, re);
    SETATTR_Complex_im(interp, d, 1 + im);
    SETATTR_Complex_re(interp, e, -re);
    SETATTR_Complex_im(interp, e, 1 - im);

    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 2917 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "ln"), "->P", &d);
#line 1499 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, d_re);
    GETATTR_Complex_im(interp, d, d_im);

    SETATTR_Complex_re(interp, e, (d_im ? d_im : -0.0) / -2.0);
    SETATTR_Complex_im(interp, e, d_re / 2.0);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 2931 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto atan_returns;
    /*END RETURN PMC *e */
    }
#line 1506 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
atan_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 2976 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acot(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 3014 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3030 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1509 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 3048 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "atan"), "->P", &e);
#line 1519 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 3056 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acot_returns;
    /*END RETURN PMC *e */
    }
#line 1520 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acot_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3101 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acsc(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 3139 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3155 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1523 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 3173 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "asin"), "->P", &e);
#line 1533 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 3181 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acsc_returns;
    /*END RETURN PMC *e */
    }
#line 1534 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acsc_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3226 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asec(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 3264 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3280 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1537 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 3298 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "acos"), "->P", &e);
#line 1547 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 3306 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto asec_returns;
    /*END RETURN PMC *e */
    }
#line 1548 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
asec_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3351 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sinh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 3389 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3405 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1571 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d, sinh(re) * cos(im));
    SETATTR_Complex_im(interp, d, im ? cosh(re) * sin(im) : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 3426 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto sinh_returns;
    /*END RETURN PMC *d */
    }
#line 1580 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
sinh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3471 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_cosh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 3509 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3525 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1583 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d, cosh(re) * cos(im));
    if (re == 0.0 || im == 0.0) {
        SETATTR_Complex_im(interp, d, 0.0);
    }
    else {
        SETATTR_Complex_im(interp, d, sinh(re) * sin(im));
    }

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 3551 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto cosh_returns;
    /*END RETURN PMC *d */
    }
#line 1597 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
cosh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3596 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_tanh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 3634 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3650 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1600 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 3662 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "sinh"), "->P", &d);
#line 1604 "./src/pmc/complex.pmc"

    #line 3666 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "cosh"), "->P", &e);
#line 1605 "./src/pmc/complex.pmc"


    Parrot_Complex_multi_i_divide_Complex(interp, d, e);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 3677 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto tanh_returns;
    /*END RETURN PMC *d */
    }
#line 1609 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
tanh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3722 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_coth(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 3760 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3776 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1612 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 3788 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "tanh"), "->P", &d);
#line 1616 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);

    SETATTR_Complex_re(interp, d, re ?  re / (re * re + im * im) : 0.0);
    SETATTR_Complex_im(interp, d, im ? -im / (re * re + im * im) : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 3802 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto coth_returns;
    /*END RETURN PMC *d */
    }
#line 1623 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
coth_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3847 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_csch(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 3885 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 3901 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1626 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 3913 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "sinh"), "->P", &d);
#line 1630 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);

    SETATTR_Complex_re(interp, d, re ?  re / (re * re + im * im) : 0.0);
    SETATTR_Complex_im(interp, d, im ? -im / (re * re + im * im) : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 3927 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto csch_returns;
    /*END RETURN PMC *d */
    }
#line 1637 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
csch_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 3972 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sech(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 4010 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4026 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1640 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 4038 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "cosh"), "->P", &d);
#line 1644 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);

    SETATTR_Complex_re(interp, d, re ?  re / (re * re + im * im) : 0.0);
    SETATTR_Complex_im(interp, d, im ? -im / (re * re + im * im) : 0.0);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *d */
    /*BEGIN GENERATED ACCESSORS */
#line 4052 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = d;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto sech_returns;
    /*END RETURN PMC *d */
    }
#line 1651 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
sech_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4097 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asinh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 4135 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4151 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1682 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, d,  im);
    SETATTR_Complex_im(interp, d, -re);

    #line 4169 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "asin"), "->P", &d);
#line 1692 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);
    SETATTR_Complex_re(interp, e, -im);
    SETATTR_Complex_im(interp, e,  re);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4182 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto asinh_returns;
    /*END RETURN PMC *e */
    }
#line 1698 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
asinh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4227 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acosh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 4265 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4281 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1701 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    #line 4294 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, pmc, CONST_STRING_GEN(interp, "acos"), "->P", &d);
#line 1706 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);
    SETATTR_Complex_re(interp, e, -im);
    SETATTR_Complex_im(interp, e,  re);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4307 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acosh_returns;
    /*END RETURN PMC *e */
    }
#line 1712 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acosh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4352 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_atanh(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 4390 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4406 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1715 "./src/pmc/complex.pmc"

    FLOATVAL re, im;
    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC * const e = pmc_new(interp, VTABLE_type(interp, pmc));

    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);
    SETATTR_Complex_re(interp, d,  im);
    SETATTR_Complex_im(interp, d, -re);

    #line 4424 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "atan"), "->P", &d);
#line 1725 "./src/pmc/complex.pmc"

    GETATTR_Complex_re(interp, d, re);
    GETATTR_Complex_im(interp, d, im);
    SETATTR_Complex_re(interp, e, -im);
    SETATTR_Complex_im(interp, e,  re);

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4437 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto atanh_returns;
    /*END RETURN PMC *e */
    }
#line 1731 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
atanh_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4482 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acoth(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 4520 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4536 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1734 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 4554 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "atanh"), "->P", &e);
#line 1744 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4562 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acoth_returns;
    /*END RETURN PMC *e */
    }
#line 1745 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acoth_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4607 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_acsch(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 4645 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4661 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1748 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 4679 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "asinh"), "->P", &e);
#line 1758 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4687 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto acsch_returns;
    /*END RETURN PMC *e */
    }
#line 1759 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
acsch_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4732 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_asech(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 4770 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4786 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1762 "./src/pmc/complex.pmc"

    PMC * const d = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC     *e;
    FLOATVAL re, im;
    GETATTR_Complex_re(interp, pmc, re);
    GETATTR_Complex_im(interp, pmc, im);

    SETATTR_Complex_re(interp, d,  re / (re * re + im * im));
    SETATTR_Complex_im(interp, d, -im / (re * re + im * im));

    #line 4804 "./src/pmc/complex.pmc"
Parrot_PCCINVOKE(interp, d, CONST_STRING_GEN(interp, "acosh"), "->P", &e);
#line 1772 "./src/pmc/complex.pmc"

    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *e */
    /*BEGIN GENERATED ACCESSORS */
#line 4812 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = e;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto asech_returns;
    /*END RETURN PMC *e */
    }
#line 1773 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
asech_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 4857 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_pow_Complex_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 1795 "./src/pmc/complex.pmc"

    PMC *l = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *log;

    dest = pmc_new(interp, VTABLE_type(interp, pmc));


    Parrot_PCCINVOKE(interp, pmc, CONST_STRING(interp, "ln"),
                "->P", &log);

    l = Parrot_Complex_multi_multiply_Complex_PMC(interp,
                log, value, l);

    Parrot_PCCINVOKE(interp, l, CONST_STRING(interp, "exp"),
                "->P", &dest);

    return dest;
#line 4880 "./src/pmc/complex.c"
}
static  PMC  *
Parrot_Complex_multi_pow_DEFAULT_PMC(PARROT_INTERP, PMC *pmc, PMC *value, PMC *dest)
{
#line 1814 "./src/pmc/complex.pmc"

    PMC *l = pmc_new(interp, VTABLE_type(interp, pmc));
    PMC *log;

    dest = pmc_new(interp, VTABLE_type(interp, pmc));

    Parrot_PCCINVOKE(interp, pmc, CONST_STRING(interp, "ln"),
                 "->P", &log);
    l = Parrot_Complex_multi_multiply_DEFAULT_PMC(interp, log, value, l);
    Parrot_PCCINVOKE(interp, l, CONST_STRING(interp, "exp"),
                 "->P", &dest);

    return dest;
#line 4899 "./src/pmc/complex.c"
}
static  void
Parrot_Complex_nci_sqrt(PARROT_INTERP, PMC *pmc)
{
#line 421 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    const INTVAL _n_regs_used[]   = { 0, 0, 0, 1 };
    opcode_t _param_indexes[]     = { 0 };
    opcode_t *_return_indexes;
    opcode_t *_current_args;
    PMC      * const _param_sig   = pmc_new(interp, enum_class_FixedIntegerArray);
    PMC      *_return_sig         = PMCNULL;

    PMC        *_caller_ctx = interp->ctx;
    PMC * const _ret_cont   = new_ret_continuation_pmc(interp, NULL);
    PMC        *_ctx        = Parrot_push_context(interp, _n_regs_used);
    PMC        *_ccont      = PMCNULL;

    VTABLE_set_integer_native(interp, _param_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _param_sig, 0, 2);

    UNUSED(_return_indexes);

    if (_caller_ctx) {
        _ccont = Parrot_pcc_get_continuation(interp, _caller_ctx);
    }
    else {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    Parrot_pcc_set_continuation(interp, _ctx, _ret_cont);
    PARROT_CONTINUATION(_ret_cont)->from_ctx       = _ctx;

    _current_args                       = interp->current_args;
    interp->current_args                = NULL;

#line 4937 "./src/pmc/complex.pmc"

#line 439 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    interp->params_signature     = _param_sig;
    parrot_pass_args(interp, _caller_ctx, _ctx, _current_args, _param_indexes,
        PARROT_PASS_PARAMS);

    if (PObj_get_FLAGS(_ccont) & SUB_FLAG_TAILCALL) {
        PObj_get_FLAGS(_ccont) &= ~SUB_FLAG_TAILCALL;
        Parrot_pcc_dec_recursion_depth(interp, _ctx);
        Parrot_pcc_set_caller_ctx(interp, _ctx, Parrot_pcc_get_caller_ctx(interp, _caller_ctx));
        interp->current_args = NULL;
    }
    /* BEGIN PARMS SCOPE */
    {
#line 4953 "./src/pmc/complex.pmc"
    PMC* pmc = CTX_REG_PMC(_ctx, 0);

#line 447 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* BEGIN PMETHOD BODY */
    {
#line 1829 "./src/pmc/complex.pmc"

    PMC * const result = pmc_new(interp, VTABLE_type(interp, pmc));
    const FLOATVAL absval = pmc->vtable->get_number(interp, pmc);
    FLOATVAL sx, sy, rx, ry;
    GETATTR_Complex_re(interp, pmc, sx);
    GETATTR_Complex_im(interp, pmc, sy);

    rx = sqrt((absval + sx) / 2);
    ry = sqrt((absval - sx) / 2);
    if (sy < 0)
        ry = -ry;
    SETATTR_Complex_re(interp, result, rx);
    SETATTR_Complex_im(interp, result, ry);
    #line 237 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    {
    /*BEGIN RETURN PMC *result */
    /*BEGIN GENERATED ACCESSORS */
#line 4978 "./src/pmc/complex.pmc"
    CTX_REG_PMC(_ctx, 0) = result;

#line 256 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
    /*END GENERATED ACCESSORS */
    {
        opcode_t _temp_return_indexes[] = { 0 };
        _return_indexes                 = _temp_return_indexes;
    }

    _return_sig = pmc_new(interp, enum_class_FixedIntegerArray);
    VTABLE_set_integer_native(interp, _return_sig, 1);
    VTABLE_set_integer_keyed_int(interp, _return_sig, 0, 2);

    goto sqrt_returns;
    /*END RETURN PMC *result */
    }
#line 1842 "./src/pmc/complex.pmc"

#line 456 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    }
    goto no_return;
    /* END PMETHOD BODY */

#line 470 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"
sqrt_returns:

    if (! _caller_ctx) {
        /* there is no point calling Parrot_ex_throw_from_c_args here, because
           PDB_backtrace can't deal with a missing to_ctx either. */
        exit_fatal(1, "No caller_ctx for continuation %p.", _ccont);
    }

    interp->returns_signature = _return_sig;
    parrot_pass_args(interp, _ctx, _caller_ctx, _return_indexes,
        Parrot_pcc_get_results(interp, _caller_ctx), PARROT_PASS_RESULTS);
#line 480 "/builddir/build/BUILD/parrot-1.6.0/tools/build/../../lib/Parrot/Pmc2c/PCCMETHOD.pm"

    /* END PARAMS SCOPE */
    }
    no_return:
    PObj_live_CLEAR(_param_sig);
    PObj_live_CLEAR(_return_sig);
    Parrot_pop_context(interp);
#line 5023 "./src/pmc/complex.c"
}
#include "pmc_default.h"

PARROT_EXPORT
VTABLE *Parrot_Complex_update_vtable(VTABLE *vt) {
    vt->absolute = Parrot_Complex_absolute;
    vt->add = Parrot_Complex_add;
    vt->add_float = Parrot_Complex_add_float;
    vt->add_int = Parrot_Complex_add_int;
    vt->clone = Parrot_Complex_clone;
    vt->divide = Parrot_Complex_divide;
    vt->divide_float = Parrot_Complex_divide_float;
    vt->divide_int = Parrot_Complex_divide_int;
    vt->get_bool = Parrot_Complex_get_bool;
    vt->get_integer = Parrot_Complex_get_integer;
    vt->get_integer_keyed = Parrot_Complex_get_integer_keyed;
    vt->get_integer_keyed_str = Parrot_Complex_get_integer_keyed_str;
    vt->get_number = Parrot_Complex_get_number;
    vt->get_number_keyed = Parrot_Complex_get_number_keyed;
    vt->get_number_keyed_int = Parrot_Complex_get_number_keyed_int;
    vt->get_number_keyed_str = Parrot_Complex_get_number_keyed_str;
    vt->get_pmc_keyed = Parrot_Complex_get_pmc_keyed;
    vt->get_pmc_keyed_int = Parrot_Complex_get_pmc_keyed_int;
    vt->get_pmc_keyed_str = Parrot_Complex_get_pmc_keyed_str;
    vt->get_string = Parrot_Complex_get_string;
    vt->i_absolute = Parrot_Complex_i_absolute;
    vt->i_add = Parrot_Complex_i_add;
    vt->i_add_float = Parrot_Complex_i_add_float;
    vt->i_add_int = Parrot_Complex_i_add_int;
    vt->i_divide = Parrot_Complex_i_divide;
    vt->i_divide_float = Parrot_Complex_i_divide_float;
    vt->i_divide_int = Parrot_Complex_i_divide_int;
    vt->i_multiply = Parrot_Complex_i_multiply;
    vt->i_multiply_float = Parrot_Complex_i_multiply_float;
    vt->i_multiply_int = Parrot_Complex_i_multiply_int;
    vt->i_neg = Parrot_Complex_i_neg;
    vt->i_subtract = Parrot_Complex_i_subtract;
    vt->i_subtract_float = Parrot_Complex_i_subtract_float;
    vt->i_subtract_int = Parrot_Complex_i_subtract_int;
    vt->init = Parrot_Complex_init;
    vt->init_pmc = Parrot_Complex_init_pmc;
    vt->invoke = Parrot_Complex_invoke;
    vt->is_equal = Parrot_Complex_is_equal;
    vt->multiply = Parrot_Complex_multiply;
    vt->multiply_float = Parrot_Complex_multiply_float;
    vt->multiply_int = Parrot_Complex_multiply_int;
    vt->neg = Parrot_Complex_neg;
    vt->pow = Parrot_Complex_pow;
    vt->set_integer_keyed = Parrot_Complex_set_integer_keyed;
    vt->set_integer_keyed_str = Parrot_Complex_set_integer_keyed_str;
    vt->set_integer_native = Parrot_Complex_set_integer_native;
    vt->set_number_keyed = Parrot_Complex_set_number_keyed;
    vt->set_number_keyed_int = Parrot_Complex_set_number_keyed_int;
    vt->set_number_keyed_str = Parrot_Complex_set_number_keyed_str;
    vt->set_number_native = Parrot_Complex_set_number_native;
    vt->set_pmc = Parrot_Complex_set_pmc;
    vt->set_pmc_keyed = Parrot_Complex_set_pmc_keyed;
    vt->set_pmc_keyed_str = Parrot_Complex_set_pmc_keyed_str;
    vt->set_string_native = Parrot_Complex_set_string_native;
    vt->subtract = Parrot_Complex_subtract;
    vt->subtract_float = Parrot_Complex_subtract_float;
    vt->subtract_int = Parrot_Complex_subtract_int;
    vt->attr_size = sizeof(Parrot_Complex_attributes);

    return vt;
}


PARROT_EXPORT
VTABLE *Parrot_Complex_ro_update_vtable(ARGMOD(VTABLE *vt)) {
    vt->absolute = Parrot_Complex_absolute;
    vt->add = Parrot_Complex_add;
    vt->add_float = Parrot_Complex_add_float;
    vt->add_int = Parrot_Complex_add_int;
    vt->clone = Parrot_Complex_clone;
    vt->divide = Parrot_Complex_divide;
    vt->divide_float = Parrot_Complex_divide_float;
    vt->divide_int = Parrot_Complex_divide_int;
    vt->get_bool = Parrot_Complex_get_bool;
    vt->get_integer = Parrot_Complex_get_integer;
    vt->get_integer_keyed = Parrot_Complex_get_integer_keyed;
    vt->get_integer_keyed_str = Parrot_Complex_get_integer_keyed_str;
    vt->get_number = Parrot_Complex_get_number;
    vt->get_number_keyed = Parrot_Complex_get_number_keyed;
    vt->get_number_keyed_int = Parrot_Complex_get_number_keyed_int;
    vt->get_number_keyed_str = Parrot_Complex_get_number_keyed_str;
    vt->get_pmc_keyed = Parrot_Complex_get_pmc_keyed;
    vt->get_pmc_keyed_int = Parrot_Complex_get_pmc_keyed_int;
    vt->get_pmc_keyed_str = Parrot_Complex_get_pmc_keyed_str;
    vt->get_string = Parrot_Complex_get_string;
    vt->init = Parrot_Complex_init;
    vt->init_pmc = Parrot_Complex_init_pmc;
    vt->invoke = Parrot_Complex_invoke;
    vt->is_equal = Parrot_Complex_is_equal;
    vt->multiply = Parrot_Complex_multiply;
    vt->multiply_float = Parrot_Complex_multiply_float;
    vt->multiply_int = Parrot_Complex_multiply_int;
    vt->neg = Parrot_Complex_neg;
    vt->pow = Parrot_Complex_pow;
    vt->subtract = Parrot_Complex_subtract;
    vt->subtract_float = Parrot_Complex_subtract_float;
    vt->subtract_int = Parrot_Complex_subtract_int;
    vt->attr_size = sizeof(Parrot_Complex_attributes);

    return vt;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
VTABLE* Parrot_Complex_get_vtable(PARROT_INTERP) {
    VTABLE *vt;
    vt = Parrot_default_get_vtable(interp);
    Parrot_Complex_update_vtable(vt);

    return vt;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
VTABLE* Parrot_Complex_ro_get_vtable(PARROT_INTERP) {
    VTABLE *vt;
    vt = Parrot_default_ro_get_vtable(interp);
    Parrot_Complex_ro_update_vtable(vt);

    return vt;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
PMC* Parrot_Complex_get_mro(PARROT_INTERP, PMC* mro) {
    if (PMC_IS_NULL(mro)) {
        mro = pmc_new(interp, enum_class_ResizableStringArray);
    }

    VTABLE_unshift_string(interp, mro,
        string_make(interp, "Complex", 7, NULL, 0));
    return mro;
}

PARROT_EXPORT
PARROT_CANNOT_RETURN_NULL
PARROT_WARN_UNUSED_RESULT
Hash* Parrot_Complex_get_isa(PARROT_INTERP, Hash* isa) {
    if (isa == NULL) {
        isa = parrot_new_hash(interp);
    }

    parrot_hash_put(interp, isa, (void *)(CONST_STRING_GEN(interp, "Complex")), PMCNULL);
    return isa;
}

void
Parrot_Complex_class_init(PARROT_INTERP, int entry, int pass)
{
    static const char attr_defs [] =
        "Fre Fim ";
    if (pass == 0) {
        VTABLE * const vt  = Parrot_Complex_get_vtable(interp);
        vt->base_type      = enum_class_Complex;
        vt->flags          = 0|VTABLE_HAS_READONLY_FLAG;
        vt->attribute_defs = attr_defs;
        interp->vtables[entry] = vt;

        vt->whoami       = CONST_STRING_GEN(interp, "Complex");
        vt->provides_str = CONST_STRING_GEN(interp, "scalar");
        vt->isa_hash     = NULL;
        {
            VTABLE                   *vt_ro;
            vt_ro                 = Parrot_Complex_ro_get_vtable(interp);
            vt_ro->base_type      = enum_class_Complex;
            vt_ro->flags          = 0|VTABLE_IS_READONLY_FLAG;

            vt_ro->attribute_defs = attr_defs;

            vt_ro->base_type           = entry;
            vt_ro->whoami              = vt->whoami;
            vt_ro->provides_str        = vt->provides_str;
            vt->ro_variant_vtable      = vt_ro;
            vt_ro->ro_variant_vtable = vt;
            vt_ro->isa_hash            = vt->isa_hash;
        }

    }
    else { /* pass */
        {
            VTABLE * const vt  = interp->vtables[entry];

            vt->mro = Parrot_Complex_get_mro(interp, PMCNULL);

            if (vt->ro_variant_vtable)
                vt->ro_variant_vtable->mro = vt->mro;
        }

        /* set up MRO and _namespace */
        Parrot_create_mro(interp, entry);
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_ln), CONST_STRING_GEN(interp, "ln"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_exp), CONST_STRING_GEN(interp, "exp"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sin), CONST_STRING_GEN(interp, "sin"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_cos), CONST_STRING_GEN(interp, "cos"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_tan), CONST_STRING_GEN(interp, "tan"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_cot), CONST_STRING_GEN(interp, "cot"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sec), CONST_STRING_GEN(interp, "sec"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_csc), CONST_STRING_GEN(interp, "csc"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asin), CONST_STRING_GEN(interp, "asin"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acos), CONST_STRING_GEN(interp, "acos"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_atan), CONST_STRING_GEN(interp, "atan"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acot), CONST_STRING_GEN(interp, "acot"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acsc), CONST_STRING_GEN(interp, "acsc"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asec), CONST_STRING_GEN(interp, "asec"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sinh), CONST_STRING_GEN(interp, "sinh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_cosh), CONST_STRING_GEN(interp, "cosh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_tanh), CONST_STRING_GEN(interp, "tanh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_coth), CONST_STRING_GEN(interp, "coth"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_csch), CONST_STRING_GEN(interp, "csch"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sech), CONST_STRING_GEN(interp, "sech"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asinh), CONST_STRING_GEN(interp, "asinh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acosh), CONST_STRING_GEN(interp, "acosh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_atanh), CONST_STRING_GEN(interp, "atanh"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acoth), CONST_STRING_GEN(interp, "acoth"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_acsch), CONST_STRING_GEN(interp, "acsch"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_asech), CONST_STRING_GEN(interp, "asech"));
        register_raw_nci_method_in_ns(interp, entry, F2DPTR(Parrot_Complex_nci_sqrt), CONST_STRING_GEN(interp, "sqrt"));
        {
        STRING * const mfl_0 = CONST_STRING_GEN(interp, "add");
        STRING * const mfl_1 = CONST_STRING_GEN(interp, "PJPPP");
        STRING * const mfl_2 = CONST_STRING_GEN(interp, "Complex,Complex,PMC");
        STRING * const mfl_3 = CONST_STRING_GEN(interp, "Complex");
        STRING * const mfl_4 = CONST_STRING_GEN(interp, "Complex,DEFAULT,PMC");
        STRING * const mfl_5 = CONST_STRING_GEN(interp, "i_add");
        STRING * const mfl_6 = CONST_STRING_GEN(interp, "vJPP");
        STRING * const mfl_7 = CONST_STRING_GEN(interp, "Complex,Complex");
        STRING * const mfl_8 = CONST_STRING_GEN(interp, "Complex,DEFAULT");
        STRING * const mfl_9 = CONST_STRING_GEN(interp, "subtract");
        STRING * const mfl_10 = CONST_STRING_GEN(interp, "i_subtract");
        STRING * const mfl_11 = CONST_STRING_GEN(interp, "multiply");
        STRING * const mfl_12 = CONST_STRING_GEN(interp, "i_multiply");
        STRING * const mfl_13 = CONST_STRING_GEN(interp, "divide");
        STRING * const mfl_14 = CONST_STRING_GEN(interp, "i_divide");
        STRING * const mfl_15 = CONST_STRING_GEN(interp, "is_equal");
        STRING * const mfl_16 = CONST_STRING_GEN(interp, "IJPP");
        STRING * const mfl_17 = CONST_STRING_GEN(interp, "pow");

             const  multi_func_list _temp_multi_func_list[] = {
                        { mfl_0,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_add_Complex_PMC }
,
        { mfl_0,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_add_DEFAULT_PMC }
,
        { mfl_5,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_add_Complex }
,
        { mfl_5,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_add_DEFAULT }
,
        { mfl_9,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_subtract_Complex_PMC }
,
        { mfl_9,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_subtract_DEFAULT_PMC }
,
        { mfl_10,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_subtract_Complex }
,
        { mfl_10,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_subtract_DEFAULT }
,
        { mfl_11,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_multiply_Complex_PMC }
,
        { mfl_11,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_multiply_DEFAULT_PMC }
,
        { mfl_12,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_multiply_Complex }
,
        { mfl_12,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_multiply_DEFAULT }
,
        { mfl_13,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_divide_Complex_PMC }
,
        { mfl_13,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_divide_DEFAULT_PMC }
,
        { mfl_14,
          mfl_6,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_divide_Complex }
,
        { mfl_14,
          mfl_6,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_i_divide_DEFAULT }
,
        { mfl_15,
          mfl_16,
          mfl_7,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_is_equal_Complex }
,
        { mfl_15,
          mfl_16,
          mfl_8,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_is_equal_DEFAULT }
,
        { mfl_17,
          mfl_1,
          mfl_2,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_pow_Complex_PMC }
,
        { mfl_17,
          mfl_1,
          mfl_4,
          mfl_3,
          (funcptr_t) Parrot_Complex_multi_pow_DEFAULT_PMC }

            };
#define N_MULTI_LIST (sizeof(_temp_multi_func_list)/sizeof(_temp_multi_func_list[0]))
            Parrot_mmd_add_multi_list_from_c_args(interp,
                _temp_multi_func_list, N_MULTI_LIST);
        }
    } /* pass */
} /* Parrot_Complex_class_init */
#line 1844 "./src/pmc/complex.pmc"


/*

=back

=cut

*/

/*
 * Local variables:
 *   c-file-style: "parrot"
 * End:
 * vim: expandtab shiftwidth=4:
 */
